"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var yaml = require("js-yaml");
var path = require("path");
var page_generator_1 = require("./page-generator");
var utils_1 = require("./utils");
function requireSettings(configFilePath) {
    var generateActualUbwConfigs = require(configFilePath);
    var actualConfigs = generateActualUbwConfigs();
    var configs = page_generator_1.fillWithDefaultUbwConfigs(actualConfigs);
    var blogRoot = path.join(path.dirname(configFilePath), configs.blogDir);
    return {
        configs: configs,
        blogRoot: blogRoot,
    };
}
exports.requireSettings = requireSettings;
exports.createDefaultUbwConfigs = page_generator_1.createDefaultUbwConfigs;
exports.cliUtils = {
    CONFIG_FILE_NAME: utils_1.CONFIG_FILE_NAME,
    toNormalizedAbsolutePath: utils_1.toNormalizedAbsolutePath,
};
//
// Executables
//
function executeArticleNew(configFilePath) {
    var _a = requireSettings(configFilePath), configs = _a.configs, blogRoot = _a.blogRoot;
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    fs.ensureDirSync(paths.sourceRoot);
    fs.ensureDirSync(paths.sourceArticlesRoot);
    var articlePages = page_generator_1.initializeArticlePages(blogRoot, configs, fs.readdirSync(paths.sourceArticlesRoot));
    var now = new Date();
    var todayDateString = utils_1.generateTodayDateString(now, configs.timeZone);
    var articleId = page_generator_1.getNextAutomaticArticleId(articlePages, todayDateString);
    var frontMatters = page_generator_1.createInitialArticleFrontMatters(articleId, utils_1.generateDateTimeString(now, 'UTC', { timeZoneSuffix: true }));
    fs.writeFileSync(path.join(paths.sourceArticlesRoot, articleId + '.md'), [
        // TODO: Want to wrap string variables with double quotes always.
        '---\n' + yaml.safeDump(frontMatters) + '---',
        '',
        '# Page Title\n',
    ].join('\n'));
    return Promise.resolve({
        exitCode: 0,
        message: '',
    });
}
exports.executeArticleNew = executeArticleNew;
function executeCompile(configFilePath) {
    var settings = requireSettings(configFilePath);
    return executeCompileWithSettings(settings);
}
exports.executeCompile = executeCompile;
// Separate it from `executeCompile` to change the `configs` at the time of the test
function executeCompileWithSettings(settings) {
    var configs = settings.configs, blogRoot = settings.blogRoot;
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    var articlePages = page_generator_1.initializeArticlePages(blogRoot, configs, fs.readdirSync(paths.sourceArticlesRoot))
        .map(function (articlePage) {
        return Object.assign({}, articlePage, {
            markdown: fs.readFileSync(articlePage.inputFilePath).toString(),
        });
    });
    var nonArticlePages = page_generator_1.initializeNonArticlePages(blogRoot, configs);
    articlePages = page_generator_1.preprocessArticlePages(blogRoot, configs, articlePages);
    nonArticlePages = page_generator_1.preprocessNonArticlePages(blogRoot, configs, nonArticlePages);
    articlePages = page_generator_1.generateArticlePages(blogRoot, configs, articlePages, nonArticlePages);
    nonArticlePages = page_generator_1.generateNonArticlePages(blogRoot, configs, articlePages, nonArticlePages);
    fs.ensureDirSync(paths.publicationRoot);
    fs.ensureDirSync(paths.publicationArticlesRoot);
    articlePages.forEach(function (article) {
        fs.writeFileSync(article.outputFilePath, article.html);
    });
    nonArticlePages.forEach(function (nonArticlePage) {
        fs.writeFileSync(nonArticlePage.outputFilePath, nonArticlePage.html);
    });
    fs.copySync(paths.sourceExternalResourcesRoot, paths.publicationExternalResourcesRoot);
    // Expand files under the "external-resources/_direct" into under the document root.
    //
    // NOTICE: Previously expanded files using "fs-extra"'s `moveSync` like the following code,
    //         this didn't work in the Travis CI environment.
    //         ```
    //         fs.moveSync("/path/to/external-resources/_direct", "/path/to/publication");
    //         ```
    fs.ensureDirSync(paths.publicationExternalResourcesDirectPlacementRoot);
    fs.readdirSync(paths.publicationExternalResourcesDirectPlacementRoot).forEach(function (fileName) {
        var from = path.join(paths.publicationExternalResourcesDirectPlacementRoot, fileName);
        var to = path.join(paths.publicationRoot, fileName);
        fs.copySync(from, to);
    });
    fs.removeSync(paths.publicationExternalResourcesDirectPlacementRoot);
    return Promise.resolve({
        exitCode: 0,
        message: '',
    });
}
exports.executeCompileWithSettings = executeCompileWithSettings;
function executeHelp() {
    return Promise.resolve({
        exitCode: 0,
        message: 'Please see the README of https://github.com/kjirou/unlimited-blog-works',
    });
}
exports.executeHelp = executeHelp;
function executeInit(blogRoot) {
    var configFilePath = path.join(blogRoot, utils_1.CONFIG_FILE_NAME);
    var initialConfigs = page_generator_1.createInitialUbwConfigs();
    var configs = page_generator_1.fillWithDefaultUbwConfigs(initialConfigs);
    var configFileSource = [
        'module.exports = function ubwConfigs() {',
        ("return " + JSON.stringify(initialConfigs, null, 2) + ";")
            .split('\n')
            .map(function (line) { return '  ' + line; })
            .join('\n'),
        '}',
        '',
    ].join('\n');
    fs.ensureDirSync(blogRoot);
    fs.writeFileSync(configFilePath, configFileSource);
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    fs.ensureDirSync(paths.sourceExternalResourcesRoot);
    fs.copySync(utils_1.PRESETS_EXTERNAL_RESOURCES_ROOT, paths.sourceExternalResourcesRoot);
    return Promise.resolve({
        exitCode: 0,
        message: '',
    });
}
exports.executeInit = executeInit;
function executeNow() {
    return Promise.resolve({
        exitCode: 0,
        message: utils_1.generateDateTimeString(new Date(), 'UTC', { timeZoneSuffix: true }),
    });
}
exports.executeNow = executeNow;
function executeVersion() {
    var packageJson = require('../package.json');
    return Promise.resolve({
        exitCode: 0,
        message: packageJson.version,
    });
}
exports.executeVersion = executeVersion;
