"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var hast = require("hastscript");
var yaml = require("js-yaml");
var path = require("path");
var React = require("react");
var ReactDOMServer = require("react-dom/server");
var urlModule = require("url");
var ArticleLayout_1 = require("./templates/ArticleLayout");
var TopLayout_1 = require("./templates/TopLayout");
var utils_1 = require("./utils");
// NOTICE: "unified" set MUST use only in the file
var rehypeAutolinkHeadings = require('rehype-autolink-headings');
var rehypeDocument = require('rehype-document');
var rehypeFormat = require('rehype-format');
var rehypeParse = require('rehype-parse');
var rehypeRaw = require('rehype-raw');
var rehypeSlug = require('rehype-slug');
var rehypeStringify = require('rehype-stringify');
var remarkFrontmatter = require('remark-frontmatter');
var remarkParse = require('remark-parse');
var remarkRehype = require('remark-rehype');
var unified = require('unified');
var unistUtilRemove = require('unist-util-remove');
var unistUtilVisit = require('unist-util-visit');
// NOTICE: Its type definition file exists but it is broken.
var Feed = require('feed').Feed;
function createDefaultUbwConfigs() {
    return {
        blogName: 'My Blog',
        blogUrl: 'https://example.com',
        blogDir: '.',
        publicationDir: './blog-publication',
        cssUrls: [
            "/" + utils_1.RELATIVE_EXTERNAL_RESOURCES_DIR_PATH + "/index.css",
        ],
        jsUrls: [],
        language: 'en',
        timeZone: 'UTC',
        ogp: true,
        defaultOgpImageUrl: '',
        additionalTopPageLinks: [
            {
                linkText: 'Powered by unlimited-blog-works',
                href: 'https://github.com/kjirou/unlimited-blog-works',
            },
        ],
        generateArticleHeadNodes: function () {
            return [];
        },
        generateNonArticleHeadNodes: function () {
            return [];
        },
        renderArticle: function (props) {
            return ReactDOMServer.renderToStaticMarkup(React.createElement(ArticleLayout_1.default, props));
        },
        nonArticles: [
            {
                nonArticlePageId: 'top',
                path: 'index.html',
                pathIsNormalizedToSlash: true,
                useLayout: true,
                render: function (props) {
                    return ReactDOMServer.renderToStaticMarkup(React.createElement(TopLayout_1.default, props));
                },
            },
            {
                nonArticlePageId: 'atom-feed',
                path: 'atom-feed.xml',
                pathIsNormalizedToSlash: false,
                useLayout: false,
                render: function (props) {
                    var feed = new Feed({
                        title: props.blogName,
                        id: props.blogUrl,
                    });
                    props.articles
                        .slice()
                        .sort(function (a, b) {
                        return b.lastUpdatedAt.getTime() - a.lastUpdatedAt.getTime();
                    })
                        .slice(0, 100)
                        .forEach(function (article) {
                        feed.addItem({
                            title: article.pageTitle,
                            id: article.permalink,
                            // The "link" field is not required as specification of Atom,
                            //   but this "feed" library requests it.
                            link: article.permalink,
                            date: article.lastUpdatedAt,
                        });
                    });
                    return feed.atom1() + '\n';
                },
            },
        ],
    };
}
exports.createDefaultUbwConfigs = createDefaultUbwConfigs;
function createInitialUbwConfigs() {
    var configs = createDefaultUbwConfigs();
    return {
        blogName: configs.blogName,
        blogUrl: configs.blogUrl,
        publicationDir: configs.publicationDir,
        cssUrls: configs.cssUrls,
        language: configs.language,
        timeZone: configs.timeZone,
    };
}
exports.createInitialUbwConfigs = createInitialUbwConfigs;
function fillWithDefaultUbwConfigs(configs) {
    return Object.assign({}, createDefaultUbwConfigs(), configs);
}
exports.fillWithDefaultUbwConfigs = fillWithDefaultUbwConfigs;
function findFirstImageUrl(node) {
    var found = '';
    unistUtilVisit(node, function (n) {
        if (n.type === 'image') {
            if (n.url) {
                found = n.url;
                return unistUtilVisit.EXIT;
            }
        }
    });
    return found;
}
function extractOgpDescription(node) {
    // TODO: This is a temporary fix for an issue which "unist-util-remove" breaks its params.
    var copied = JSON.parse(JSON.stringify(node));
    var filtered = unistUtilRemove(copied, function (n) { return (n.type === 'yaml' ||
        n.type === 'heading' && n.depth === 1 ||
        // NOTE: link を除外しているのは、Slack がチャットでそれを展開してしまうため。
        n.type === 'link' ||
        n.type === 'html' ||
        n.type === 'code'); });
    if (filtered === null) {
        return '';
    }
    var words = [];
    unistUtilVisit(filtered, { type: 'text' }, function (n) {
        if (n.value) {
            words.push(n.value);
        }
    });
    var collapsed = words.join(' ')
        .replace(/\s+/g, ' ');
    // NOTE: This is a value based on a miscellaneous investigation.
    var recommendedMaxLength = 90;
    return collapsed.length <= recommendedMaxLength
        ? collapsed
        : collapsed.slice(0, recommendedMaxLength - 3) + '...';
}
exports.extractOgpDescription = extractOgpDescription;
function generateOgpNodes(title, image, url, siteName, description) {
    return [
        hast('meta', { property: 'og:title', content: title }),
        hast('meta', { property: 'og:type', content: 'website' })
    ].concat((image !== '' ? [hast('meta', { property: 'og:image', content: image })] : []), [
        hast('meta', { property: 'og:url', content: url }),
        hast('meta', { property: 'og:site_name', content: siteName })
    ], (description !== '' ? [hast('meta', { property: 'og:description', content: description })] : []));
}
function createRemarkPlugins() {
    return [
        [remarkFrontmatter, ['yaml']],
    ];
}
/**
 * Generate an unified's transformer that empty a href of <h1>'s autolink
 */
function generateH1AutolinkHrefReplacementTransformer(autolinkMarkerAttributeName) {
    return function transformer(tree) {
        unistUtilVisit(tree, { type: 'element', tagName: 'h1' }, function (h1Node) {
            unistUtilVisit(h1Node, { type: 'element', tagName: 'a' }, function (anchorNode) {
                if (anchorNode.properties && anchorNode.properties[autolinkMarkerAttributeName] === true) {
                    // NOTE: The empty string will probably work except for IE(<= 10)
                    // Ref) https://hail2u.net/blog/coding/empty-href-value.html
                    anchorNode.properties.href = '';
                }
            });
        });
    };
}
exports.generateH1AutolinkHrefReplacementTransformer = generateH1AutolinkHrefReplacementTransformer;
;
function createRehypePlugins(params) {
    var documentOptions = {
        title: params.title,
        language: params.language,
        css: params.cssUrls,
    };
    documentOptions.css = params.cssUrls;
    documentOptions.js = params.jsUrls;
    var additionalHeadNodes = params.additionalHeadNodes;
    var autolinkContent = {
        type: 'text',
        value: '#',
    };
    return [
        [rehypeRaw],
        [rehypeSlug],
        [rehypeAutolinkHeadings, {
                behavior: 'append',
                content: autolinkContent,
                properties: {
                    className: 'ubw-heading-slug',
                    ariaHidden: true,
                    // NOTICE: Apply to search with `unist-util-*`. It is not used in HTML.
                    dataUbwAutolink: true,
                },
            }],
        function () {
            return generateH1AutolinkHrefReplacementTransformer('dataUbwAutolink');
        },
        [rehypeDocument, documentOptions],
        function () {
            return function transformer(tree) {
                unistUtilVisit(tree, { type: 'element', tagName: 'head' }, function (node) {
                    params.additionalHeadNodes.forEach(function (nodeInHead) {
                        (node.children || []).push(nodeInHead);
                    });
                });
            };
        },
        [rehypeFormat],
    ];
}
function createDefaultArticleFrontMatters() {
    return {
        lastUpdatedAt: '',
        publicId: '',
    };
}
function createInitialArticleFrontMatters(publicId, lastUpdatedAt) {
    return {
        publicId: publicId,
        lastUpdatedAt: lastUpdatedAt,
    };
}
exports.createInitialArticleFrontMatters = createInitialArticleFrontMatters;
function fillWithDefaultArticleFrontMatters(actualFrontMatters) {
    return Object.assign({}, createDefaultArticleFrontMatters(), actualFrontMatters);
}
function createArticlePage() {
    return {
        articleId: '',
        publicId: '',
        inputFilePath: '',
        outputFilePath: '',
        rootRelativePath: '',
        permalink: '',
        ogpImageUrl: '',
        ogpDescription: '',
        html: '',
        markdown: '',
        pageTitle: '',
        lastUpdatedAt: new Date(1970, 0, 1),
    };
}
exports.createArticlePage = createArticlePage;
function initializeArticlePages(blogRoot, configs, articleFileNames) {
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    return articleFileNames.map(function (articleFileName) {
        var articleFilePath = path.join(paths.sourceArticlesRoot, articleFileName);
        return Object.assign(createArticlePage(), {
            articleId: path.basename(articleFilePath, '.md'),
            inputFilePath: articleFilePath,
        });
    });
}
exports.initializeArticlePages = initializeArticlePages;
// e.g. "20191231-0001"
// Each part is called "{dateString}-{serialString}".
var AUTOMATIC_ARTICLE_ID = /^(\d{8})-(\d{4})$/;
function getNextAutomaticArticleId(articlePages, dateString) {
    var lastSerial = 0;
    articlePages.forEach(function (articlePage) {
        var matched = AUTOMATIC_ARTICLE_ID.exec(articlePage.articleId);
        if (matched !== null && matched[1] === dateString) {
            var foundSerial = parseInt(matched[2]);
            if (foundSerial >= lastSerial) {
                lastSerial = foundSerial;
            }
        }
    });
    return dateString + "-" + (lastSerial + 1).toString().padStart(4, '0');
}
exports.getNextAutomaticArticleId = getNextAutomaticArticleId;
function preprocessArticlePages(blogRoot, configs, articlePages) {
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    // NOTE: .html を生成する際にも .md を Ast にする変換は行なっており、つまりは二度同じことをしている。
    //       これは、unified().parse() で生成した Ast を再利用して .stringify() へ直接渡す方法が不明だったため。
    return articlePages.map(function (articlePage) {
        var ast = unified()
            .use(remarkParse)
            .use(createRemarkPlugins())
            .parse(articlePage.markdown);
        var frontMattersNode = ast.children[0];
        if (frontMattersNode.type !== 'yaml') {
            throw new Error('Can not find a Front-matter block in an articlePage.');
        }
        var actualFrontMatters = yaml.safeLoad(frontMattersNode.value);
        var frontMatters = fillWithDefaultArticleFrontMatters(actualFrontMatters);
        var basePath = utils_1.getPathnameWithoutTailingSlash(configs.blogUrl);
        var rootRelativePath = basePath + "/" + utils_1.RELATIVE_ARTICLES_DIR_PATH + "/" + frontMatters.publicId + ".html";
        var permalink = configs.blogUrl + "/" + utils_1.RELATIVE_ARTICLES_DIR_PATH + "/" + frontMatters.publicId + ".html";
        var ogpImageUrl = '';
        if (configs.ogp) {
            ogpImageUrl = configs.defaultOgpImageUrl;
            var foundUrl = findFirstImageUrl(ast);
            if (foundUrl) {
                var urlType = utils_1.classifyUrl(foundUrl);
                if (urlType === 'absolute') {
                    ogpImageUrl = foundUrl;
                }
                else if (urlType === 'root-relative') {
                    ogpImageUrl = "" + configs.blogUrl + foundUrl;
                }
                else if (urlType === 'relative') {
                    ogpImageUrl = urlModule.resolve(configs.blogUrl + "/" + utils_1.RELATIVE_ARTICLES_DIR_PATH + "/", foundUrl);
                }
            }
        }
        var ogpDescription = extractOgpDescription(ast);
        return Object.assign({}, articlePage, {
            outputFilePath: path.join(paths.publicationArticlesRoot, frontMatters.publicId + '.html'),
            rootRelativePath: rootRelativePath,
            permalink: permalink,
            ogpImageUrl: ogpImageUrl,
            ogpDescription: ogpDescription,
            pageTitle: utils_1.extractPageTitle(ast),
            lastUpdatedAt: new Date(frontMatters.lastUpdatedAt),
        });
    });
}
exports.preprocessArticlePages = preprocessArticlePages;
function generateArticlePages(blogRoot, configs, articlePages, nonArticlePages) {
    var nonArticlesProps = nonArticlePages.reduce(function (summary, nonArticlePage) {
        var _a;
        return Object.assign({}, summary, (_a = {},
            _a[nonArticlePage.nonArticlePageId] = {
                permalink: nonArticlePage.permalink,
                rootRelativePath: nonArticlePage.rootRelativePath,
            },
            _a));
    }, {});
    return articlePages.map(function (articlePage) {
        var contentHtmlData = unified()
            .use(remarkParse)
            .use(createRemarkPlugins())
            .use(remarkRehype, {
            allowDangerousHTML: true,
        })
            .use(rehypeStringify)
            .processSync(articlePage.markdown);
        var articlePageProps = {
            blogName: configs.blogName,
            blogUrl: configs.blogUrl,
            contentHtml: contentHtmlData.contents,
            lastUpdatedAt: articlePage.lastUpdatedAt,
            formattedLastUpdatedAt: utils_1.generateDateTimeString(articlePage.lastUpdatedAt, configs.timeZone),
            timeZone: configs.timeZone,
            nonArticles: nonArticlesProps,
        };
        var articleHtml = configs.renderArticle(articlePageProps);
        var ogpNodes = configs.ogp
            ? generateOgpNodes(articlePage.pageTitle, articlePage.ogpImageUrl, articlePage.permalink, configs.blogName, articlePage.ogpDescription)
            : [];
        var unifiedResult = unified()
            .use(rehypeParse, {
            fragment: true,
        })
            .use(createRehypePlugins({
            title: articlePage.pageTitle + " | " + configs.blogName,
            language: configs.language,
            cssUrls: configs.cssUrls,
            jsUrls: configs.jsUrls,
            additionalHeadNodes: ogpNodes.concat(configs.generateArticleHeadNodes(articlePageProps)),
        }))
            .use(rehypeStringify)
            .processSync(articleHtml);
        return Object.assign({}, articlePage, {
            html: unifiedResult.contents,
        });
    });
}
exports.generateArticlePages = generateArticlePages;
function initializeNonArticlePages(blogRoot, configs) {
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    var basePath = utils_1.getPathnameWithoutTailingSlash(configs.blogUrl);
    return configs.nonArticles.map(function (nonArticleConfigs) {
        var blogRootRelativePath = nonArticleConfigs.pathIsNormalizedToSlash
            ? utils_1.removeTailingResourceNameFromPath(nonArticleConfigs.path)
            : nonArticleConfigs.path;
        return {
            nonArticlePageId: nonArticleConfigs.nonArticlePageId,
            render: nonArticleConfigs.render,
            rootRelativePath: basePath + '/' + blogRootRelativePath,
            permalink: configs.blogUrl + '/' + blogRootRelativePath,
            outputFilePath: path.join(paths.publicationRoot, nonArticleConfigs.path),
            useLayout: nonArticleConfigs.useLayout,
            html: '',
        };
    });
}
exports.initializeNonArticlePages = initializeNonArticlePages;
function preprocessNonArticlePages(blogRoot, configs, nonArticlePages) {
    return nonArticlePages;
}
exports.preprocessNonArticlePages = preprocessNonArticlePages;
function generateNonArticlePages(blogRoot, configs, articlePages, nonArticlePages) {
    var paths = utils_1.generateBlogPaths(blogRoot, configs.publicationDir);
    var articlesProps = articlePages.map(function (articlePage) {
        return {
            articleId: articlePage.articleId,
            lastUpdatedAt: articlePage.lastUpdatedAt,
            formattedLastUpdatedAt: utils_1.generateDateTimeString(articlePage.lastUpdatedAt, configs.timeZone),
            pageTitle: articlePage.pageTitle,
            permalink: articlePage.permalink,
            rootRelativePath: articlePage.rootRelativePath,
        };
    });
    var nonArticlesProps = nonArticlePages.map(function (nonArticlePage) {
        return {
            id: nonArticlePage.nonArticlePageId,
            permalink: nonArticlePage.permalink,
            rootRelativePath: nonArticlePage.rootRelativePath,
        };
    });
    return nonArticlePages.map(function (nonArticlePage) {
        var nonArticlePageProps = {
            additionalTopPageLinks: configs.additionalTopPageLinks,
            articles: articlesProps,
            blogName: configs.blogName,
            blogUrl: configs.blogUrl,
            nonArticles: nonArticlesProps,
            permalink: nonArticlePage.permalink,
            rootRelativePath: nonArticlePage.rootRelativePath,
            timeZone: configs.timeZone,
        };
        var html = nonArticlePage.render(nonArticlePageProps);
        if (nonArticlePage.useLayout) {
            var ogpNodes = configs.ogp
                ? generateOgpNodes(configs.blogName, configs.defaultOgpImageUrl, nonArticlePage.permalink, configs.blogName, '')
                : [];
            var unifiedResult = unified()
                .use(rehypeParse, {
                fragment: true,
            })
                .use(createRehypePlugins({
                title: configs.blogName,
                language: configs.language,
                cssUrls: configs.cssUrls,
                jsUrls: configs.jsUrls,
                additionalHeadNodes: ogpNodes.concat(configs.generateNonArticleHeadNodes(nonArticlePageProps))
            }))
                .use(rehypeStringify)
                .processSync(html);
            html = unifiedResult.contents;
        }
        return Object.assign({}, nonArticlePage, {
            html: html,
        });
    });
}
exports.generateNonArticlePages = generateNonArticlePages;
